from blueness import module
from blue_geo import NAME
from blue_geo.logger import logger


NAME = module.name(__file__, NAME)


def reduce_function(object_name: str) -> bool:
    logger.info(f"{NAME}.reduce: {object_name}")

    logger.info("🪄")

    return True



import argparse
from blueness import module
from blue_geo import NAME, VERSION
from blue_geo.catalog import get_datacube
from blue_geo.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="get|ingest",
)
parser.add_argument(
    "--what",
    default="",
    type=str,
    help="get:catalog|template, ingest: all|metadata|quick|<suffix>",
)
parser.add_argument(
    "--datacube_id",
    type=str,
    default="",
)
parser.add_argument(
    "--catalog",
    type=str,
    default="",
)
parser.add_argument(
    "--dryrun",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--overwrite",
    default=0,
    type=int,
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "get":
    success = True
    datacube = get_datacube(datacube_id=args.datacube_id)

    output = f"unknown-{args.what}"
    if args.what == "template":
        output = datacube.QGIS_template
    elif args.what == "catalog":
        output = datacube.catalog.name

    print(output)
elif args.task == "ingest":
    datacube = get_datacube(datacube_id=args.datacube_id)
    success, _ = datacube.ingest(
        dryrun=args.dryrun == 1,
        overwrite=args.overwrite == 1,
        what=args.what if args.what else "metadata",
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from blueness import module
from blue_geo import NAME
from blue_geo.logger import logger


NAME = module.name(__file__, NAME)


def map_function(
    datacube_id: str,
    object_name: str,
) -> bool:
    logger.info(f"{NAME}.map: {datacube_id} -> {object_name}")

    logger.info("🪄")

    return True



import copy
from typing import Dict, List
from blue_options.options import Options
from abcli.file.load import load_yaml


class Target:
    def __init__(
        self,
        name: str = "",
        catalog: str = "",
        collection: str = "",
        args: Dict[str, str] = {},
    ) -> None:
        self.name: str = name

        self.catalog = catalog
        self.collection = collection

        self.args = copy.deepcopy(args)

    def __repr__(self) -> str:
        return "{}[{}]: {}/{}: {}".format(
            self.__class__.__name__,
            self.name,
            self.catalog,
            self.collection,
            self.args_as_str(" | "),
        )

    def args_as_str(self, delim: str = " ") -> str:
        return delim.join(
            [f"--{argument} {value}" for argument, value in self.args.items()]
        )


class TargetList:
    def __init__(self, filename: str = "") -> None:
        self.targets: Dict[str, Target] = {}

        if filename:
            self.load(filename)

    def load(self, filename: str) -> bool:
        self.targets = {}

        success, targets = load_yaml(filename, civilized=True)

        for target_name, target_info in targets.items():
            self.targets[target_name] = Target(
                name=target_name,
                catalog=target_info["catalog"],
                collection=target_info["collection"],
                args=target_info["args"],
            )

        return success



from blueness import module
from abcli.plugins.metadata import get_from_object
from notebooks_and_scripts.workflow.generic import Workflow
from blue_geo import NAME
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


def generate_workflow(
    query_object_name: str,
    job_name: str,
    object_name: str,
    map_options: str,
    reduce_options: str,
) -> bool:
    list_of_datacube_id = get_from_object(
        query_object_name,
        "datacube_id",
    )

    logger.info(
        "{}.generate_workflow: {}[{} X]: -[{} @ {} + {}]-> {}".format(
            NAME,
            query_object_name,
            len(list_of_datacube_id),
            map_options,
            reduce_options,
            job_name,
            object_name,
        )
    )

    workflow = Workflow(job_name)

    workflow.G.add_node("reduction")
    workflow.G.nodes["reduction"]["command_line"] = " ".join(
        [
            "workflow monitor",
            "node=reduction",
            job_name,
            "blue_geo_watch_reduce",
            reduce_options,
            object_name,
        ]
    )

    for datacube_id in list_of_datacube_id:
        workflow.G.add_node(datacube_id)
        workflow.G.nodes[datacube_id]["command_line"] = " ".join(
            [
                "workflow monitor",
                f"node={datacube_id}",
                job_name,
                "blue_geo_watch_map",
                map_options,
                datacube_id,
                object_name,
            ]
        )
        workflow.G.add_edge("reduction", datacube_id)

    return workflow.save()
